@@ -80,8 +80,8 @@ def euler_equation_solver(guesses, *args):
         theta,
         p.e[:, j],
         p.rho,
-        p.etr_params[-1, :, :],
-        p.mtry_params[-1, :, :],
+        p.etr_params[-1],
+        p.mtry_params[-1],
         None,
         j,
         p,
@@ -101,8 +101,8 @@ def euler_equation_solver(guesses, *args):
         theta,
         p.chi_n,
         p.e[:, j],
-        p.etr_params[-1, :, :],
-        p.mtrx_params[-1, :, :],
+        p.etr_params[-1],
+        p.mtrx_params[-1],
         None,
         j,
         p,
@@ -139,7 +139,7 @@ def euler_equation_solver(guesses, *args):
         False,
         "SS",
         p.e[:, j],
-        p.etr_params[-1, :, :],
+        p.etr_params[-1],
         p,
     )
     cons = household.get_cons(
@@ -270,10 +270,14 @@ def inner_loop(outer_loop_vars, p, client):
 
     theta = tax.replacement_rate_vals(nssmat, w, factor, None, p)
 
-    etr_params_3D = np.tile(
-        np.reshape(p.etr_params[-1, :, :], (p.S, 1, p.etr_params.shape[2])),
-        (1, p.J, 1),
-    )
+    num_params = len(p.etr_params[-1][0])
+    etr_params_3D = [
+        [
+            [p.etr_params[-1][s][i] for i in range(num_params)]
+            for j in range(p.J)
+        ]
+        for s in range(p.S)
+    ]
 
     net_tax = tax.net_taxes(
         r_p,
@@ -398,10 +402,15 @@ def inner_loop(outer_loop_vars, p, client):
     new_p_i = np.dot(p.io_matrix, new_p_m)
     new_p_tilde = aggr.get_ptilde(new_p_i, p.tau_c[-1, :], p.alpha_c)
 
-    etr_params_3D = np.tile(
-        np.reshape(p.etr_params[-1, :, :], (p.S, 1, p.etr_params.shape[2])),
-        (1, p.J, 1),
-    )
+    num_params = len(p.etr_params[-1][0])
+    etr_params_3D = [
+        [
+            [p.etr_params[-1][s][i] for i in range(num_params)]
+            for j in range(p.J)
+        ]
+        for s in range(p.S)
+    ]
+
     taxss = tax.net_taxes(
         new_r_p,
         new_w,
@@ -710,18 +719,29 @@ def SS_solver(
     theta = tax.replacement_rate_vals(nssmat, wss, factor_ss, None, p)
 
     # Compute effective and marginal tax rates for all agents
-    etr_params_3D = np.tile(
-        np.reshape(p.etr_params[-1, :, :], (p.S, 1, p.etr_params.shape[2])),
-        (1, p.J, 1),
-    )
-    mtrx_params_3D = np.tile(
-        np.reshape(p.mtrx_params[-1, :, :], (p.S, 1, p.mtrx_params.shape[2])),
-        (1, p.J, 1),
-    )
-    mtry_params_3D = np.tile(
-        np.reshape(p.mtry_params[-1, :, :], (p.S, 1, p.mtry_params.shape[2])),
-        (1, p.J, 1),
-    )
+    num_params = len(p.etr_params[-1][0])
+    etr_params_3D = [
+        [
+            [p.etr_params[-1][s][i] for i in range(num_params)]
+            for j in range(p.J)
+        ]
+        for s in range(p.S)
+    ]
+    mtrx_params_3D = [
+        [
+            [p.mtrx_params[-1][s][i] for i in range(num_params)]
+            for j in range(p.J)
+        ]
+        for s in range(p.S)
+    ]
+    mtry_params_3D = [
+        [
+            [p.mtry_params[-1][s][i] for i in range(num_params)]
+            for j in range(p.J)
+        ]
+        for s in range(p.S)
+    ]
+
     labor_noncompliance_rate_2D = np.tile(
         np.reshape(p.labor_income_tax_noncompliance_rate[-1, :], (1, p.J)),
         (p.S, 1),
