@@ -156,8 +156,8 @@ def firstdoughnutring(
         theta[j],
         p.e[-1, j],
         p.rho[-1],
-        p.etr_params[0, -1, :],
-        p.mtry_params[0, -1, :],
+        p.etr_params[0][-1],
+        p.mtry_params[0][-1],
         None,
         j,
         p,
@@ -178,8 +178,8 @@ def firstdoughnutring(
         theta[j],
         p.chi_n[-1],
         p.e[-1, j],
-        p.etr_params[0, -1, :],
-        p.mtrx_params[0, -1, :],
+        p.etr_params[0][-1],
+        p.mtrx_params[0][-1],
         None,
         j,
         p,
@@ -231,12 +231,12 @@ def twist_doughnut(
         j (int): index of ability type
         s (int): years of life remaining
         t (int): model period
-        etr_params (Numpy array): ETR function parameters,
-            size = sxsxnum_params
-        mtrx_params (Numpy array): labor income MTR function parameters,
-            size = sxsxnum_params
-        mtry_params (Numpy array): capital income MTR function
-            parameters, size = sxsxnum_params
+        etr_params (list): ETR function parameters,
+            list of lists with size = sxsxnum_params
+        mtrx_params (list): labor income MTR function parameters,
+            list of lists with size = sxsxnum_params
+        mtry_params (list): capital income MTR function
+            parameters, lists of lists with size = sxsxnum_params
         initial_b (Numpy array): savings of agents alive at T=0,
             size = SxJ
         p (OG-Core Specifications object): model parameters
@@ -408,22 +408,29 @@ def inner_loop(guesses, outer_loop_vars, initial_values, ubi, j, ind, p):
         tr_to_use = np.diag(tr[: p.S, :, j], p.S - (s + 2))
         ubi_to_use = np.diag(ubi[: p.S, :, j], p.S - (s + 2))
 
-        length_diag = np.diag(p.etr_params[: p.S, :, 0], p.S - (s + 2)).shape[
-            0
+        num_params = len(p.etr_params[0][0])
+        temp_etr = [
+            [p.etr_params[t][p.S - s - 2 + t][i] for i in range(num_params)]
+            for t in range(s + 2)
         ]
-        etr_params_to_use = np.zeros((length_diag, p.etr_params.shape[2]))
-        mtrx_params_to_use = np.zeros((length_diag, p.mtrx_params.shape[2]))
-        mtry_params_to_use = np.zeros((length_diag, p.mtry_params.shape[2]))
-        for i in range(p.etr_params.shape[2]):
-            etr_params_to_use[:, i] = np.diag(
-                p.etr_params[: p.S, :, i], p.S - (s + 2)
-            )
-            mtrx_params_to_use[:, i] = np.diag(
-                p.mtrx_params[: p.S, :, i], p.S - (s + 2)
-            )
-            mtry_params_to_use[:, i] = np.diag(
-                p.mtry_params[: p.S, :, i], p.S - (s + 2)
-            )
+        etr_params_to_use = [
+            [temp_etr[i][j] for j in range(num_params)] for i in range(s + 2)
+        ]
+        temp_mtrx = [
+            [p.mtrx_params[t][p.S - s - 2 + t][i] for i in range(num_params)]
+            for t in range(s + 2)
+        ]
+        mtrx_params_to_use = [
+            [temp_mtrx[i][j] for j in range(num_params)] for i in range(s + 2)
+        ]
+        temp_mtry = [
+            [p.mtry_params[t][p.S - s - 2 + t][i] for i in range(num_params)]
+            for t in range(s + 2)
+        ]
+        mtry_params_to_use = [
+            [temp_mtry[i][j] for j in range(num_params)] for i in range(s + 2)
+        ]
+
         solutions = opt.root(
             twist_doughnut,
             list(b_guesses_to_use) + list(n_guesses_to_use),
@@ -463,19 +470,19 @@ def inner_loop(guesses, outer_loop_vars, initial_values, ubi, j, ind, p):
         ubi_to_use = np.diag(ubi[t : t + p.S, :, j])
 
         # initialize array of diagonal elements
-        length_diag = np.diag(p.etr_params[t : t + p.S, :, 0]).shape[0]
-        etr_params_to_use = np.zeros((length_diag, p.etr_params.shape[2]))
-        mtrx_params_to_use = np.zeros((length_diag, p.mtrx_params.shape[2]))
-        mtry_params_to_use = np.zeros((length_diag, p.mtry_params.shape[2]))
-
-        for i in range(p.etr_params.shape[2]):
-            etr_params_to_use[:, i] = np.diag(p.etr_params[t : t + p.S, :, i])
-            mtrx_params_to_use[:, i] = np.diag(
-                p.mtrx_params[t : t + p.S, :, i]
-            )
-            mtry_params_to_use[:, i] = np.diag(
-                p.mtry_params[t : t + p.S, :, i]
-            )
+        num_params = len(p.etr_params[t][0])
+        etr_params_to_use = [
+            [p.etr_params[t + s][s][i] for i in range(num_params)]
+            for s in range(p.S)
+        ]
+        mtrx_params_to_use = [
+            [p.mtrx_params[t + s][s][i] for i in range(num_params)]
+            for s in range(p.S)
+        ]
+        mtry_params_to_use = [
+            [p.mtry_params[t + s][s][i] for i in range(num_params)]
+            for s in range(p.S)
+        ]
 
         solutions = opt.root(
             twist_doughnut,
@@ -753,12 +760,18 @@ def run_TPI(p, client=None):
         bmat_splus1 = np.zeros((p.T, p.S, p.J))
         bmat_splus1[:, :, :] = b_mat[: p.T, :, :]
 
-        etr_params_4D = np.tile(
-            p.etr_params[: p.T, :, :].reshape(
-                p.T, p.S, 1, p.etr_params.shape[2]
-            ),
-            (1, 1, p.J, 1),
-        )
+        num_params = len(p.etr_params[0][0])
+        etr_params_4D = [
+            [
+                [
+                    [p.etr_params[t][s][i] for i in range(num_params)]
+                    for j in range(p.J)
+                ]
+                for s in range(p.S)
+            ]
+            for t in range(p.T)
+        ]
+
         bqmat = household.get_bq(BQ, None, p, "TPI")
         trmat = household.get_tr(TR, None, p, "TPI")
         tax_mat = tax.net_taxes(
@@ -1087,18 +1100,37 @@ def run_TPI(p, client=None):
         print("\tDistance:", TPIdist)
 
     # Compute effective and marginal tax rates for all agents
-    mtrx_params_4D = np.tile(
-        p.mtrx_params[: p.T, :, :].reshape(
-            p.T, p.S, 1, p.mtrx_params.shape[2]
-        ),
-        (1, 1, p.J, 1),
-    )
-    mtry_params_4D = np.tile(
-        p.mtry_params[: p.T, :, :].reshape(
-            p.T, p.S, 1, p.mtry_params.shape[2]
-        ),
-        (1, 1, p.J, 1),
-    )
+    num_params = len(p.mtrx_params[0][0])
+    etr_params_4D = [
+        [
+            [
+                [p.etr_params[t][s][i] for i in range(num_params)]
+                for j in range(p.J)
+            ]
+            for s in range(p.S)
+        ]
+        for t in range(p.T)
+    ]
+    mtrx_params_4D = [
+        [
+            [
+                [p.mtrx_params[t][s][i] for i in range(num_params)]
+                for j in range(p.J)
+            ]
+            for s in range(p.S)
+        ]
+        for t in range(p.T)
+    ]
+    mtry_params_4D = [
+        [
+            [
+                [p.mtry_params[t][s][i] for i in range(num_params)]
+                for j in range(p.J)
+            ]
+            for s in range(p.S)
+        ]
+        for t in range(p.T)
+    ]
     labor_noncompliance_rate_3D = np.tile(
         np.reshape(
             p.labor_income_tax_noncompliance_rate[: p.T, :], (p.T, 1, p.J)
